@@ -395,13 +395,16 @@
       {
          con$connection <- "file"
         # con$fname <- paste0(fname,".unpacked~")
-         con$fname <- paste0(fname,".unpacked",.maketmp(),"~")
+         fbase <- .maketmp()
+         con$fname <- file.path(dirname(fbase)
+                               ,paste0(basename(fname)
+                                      ,".unpacked",basename(fbase),"~"))
          if (FALSE) {
             system(paste("gzip -f -d -k",fname.gz))
             file.rename(fname,con$fname)
          }
          else  ## without "-k" key
-            system2("gzip",c("-f -d -c",.dQuote(fname.gz)),stdout=con$fname)
+            system2("gzip",c("-f -d -c",.dQuote(fname.gz)),stdout=con$fname,stderr=FALSE)
          con$compress <- -1L
       }
       fname.aux <- paste0(fname,".aux.xml")
@@ -421,8 +424,14 @@
          else if (decompress) {
             if (verbose)
                message("local unpack")
-            con$fname <- paste0(fname,".unpacked",.maketmp(),"~")
-            system2("gzip",c("-f -d -c",.dQuote(fname.envigz)),stdout=con$fname)
+           # con$fname <- paste0(fname,".unpacked",.maketmp(),"~")
+            fbase <- .maketmp()
+            con$fname <- file.path(dirname(fbase)
+                                  ,paste0(basename(fname)
+                                         ,".unpacked",basename(fbase),"~"))
+           # print(con$fname)
+           # q()
+            system2("gzip",c("-f -d -c",.dQuote(fname.envigz)),stdout=con$fname,stderr=FALSE)
             solved <- !is.null(con$fname)
          }
          if (solved) {
@@ -452,13 +461,17 @@
       {
          con$connection <- "file"
         # con$fname <- paste0(fname,".unpacked~")
-         con$fname <- paste0(fname,".unpacked",.maketmp(),"~")
+        # con$fname <- paste0(fname,".unpacked",basename(.maketmp()),"~")
+         fbase <- .maketmp()
+         con$fname <- file.path(dirname(fbase)
+                               ,paste0(basename(fname)
+                                      ,".unpacked",basename(fbase),"~"))
          if (FALSE) {
             system(paste("gzip -f -d -k -Sgz",fname.bingz))
             file.rename(fname.bin,con$fname)
          }
          else ## without "-k" key
-            system2("gzip",c("-f -d -c",.dQuote(fname.bingz)),stdout=con$fname)
+            system2("gzip",c("-f -d -c",.dQuote(fname.bingz)),stdout=con$fname,stderr=FALSE)
          con$compress <- -1L
       }
       fname.aux <- paste0(fname.bin,".aux.xml")
@@ -474,7 +487,11 @@
       {
          con$connection <- "file"
         # con$fname <- paste0(fname,".unpacked~")
-         con$fname <- paste0(fname,".unpacked",.maketmp(),"~")
+        # con$fname <- paste0(fname,".unpacked",basename(.maketmp()),"~")
+         fbase <- .maketmp()
+         con$fname <- file.path(dirname(fbase)
+                               ,paste0(basename(fname)
+                                      ,".unpacked",basename(fbase),"~"))
          if (FALSE) ## should be obsolete
             shell(paste("bzip2 -d -c",fname.bz,"1>",con$fname))
          else {
@@ -496,7 +513,11 @@
       {
          con$connection <- "file"
         # con$fname <- paste0(fname,".unpacked~")
-         con$fname <- paste0(fname,".unpacked",.maketmp(),"~")
+        # con$fname <- paste0(fname,".unpacked",basename(.maketmp()),"~")
+         fbase <- .maketmp()
+         con$fname <- file.path(dirname(fbase)
+                               ,paste0(basename(fname)
+                                      ,".unpacked",basename(fbase),"~"))
          if (FALSE) ## should be obsolete
             shell(paste("xz -d -c",fname.xz,"1>",con$fname))
          else {
@@ -526,8 +547,11 @@
    metadata <- if ((!is.na(fname.aux)&&(file.exists(fname.aux))))
       readLines(fname.aux,warn=FALSE) else NULL
   # md <- xml2::as_list(xml2::read_xml(fname.aux))
-   if (!is.na(con$connection))
-      con$handle <- do.call(con$connection,list(con$fname,"r+b"))
+   if (!is.na(con$connection)) {
+      con$handle <- try(do.call(con$connection,list(con$fname,"r+b")),silent=TRUE)
+      if (inherits(con$handle,"try-error")) ## read-only
+         con$handle <- do.call(con$connection,list(con$fname,"rb"))
+   }
    if (("bzfile" %in% class(con$handle))||("xzfile" %in% class(con$handle)))
       con$seek <- FALSE
    else
@@ -552,9 +576,12 @@
    if (!is.null(metadata))
    {
       a <- .grep("NoDataValue",metadata,value=TRUE)
-      if (length(a))
+      if (length(a)) {
          con$nodata <- as.numeric(.gsub2("<NoDataValue.*>(.+)</NoDataValue>"
                                           ,"\\1",a[1]))
+         if (is.infinite(con$nodata)) ## "-1.79769313486232Ee308"
+            con$nodata <- sign(con$nodata)*1.7976931348623e+308
+      }
    }
    if (is.null(grid$proj4))
       grid$proj4 <- ""
@@ -568,14 +595,22 @@
      # (!("package:rgdal" %in% search()))) { 
       if ((nchar(Sys.which("gdalsrsinfo")))&&
           (!any(c("rgdal","sf") %in% loadedNamespaces()))) {
-         tmp <- .maketmp()
-         wktin <- paste0(tmp,".prj~")
-         writeLines(wkt,wktin)
-         wktout <- paste0(tmp,".wkt~")
-         system2("gdalsrsinfo",list("-o proj4",wktin),stdout=wktout)
-         grid$proj4 <- .gsub("\'","",readLines(wktout,warn=FALSE))
-         file.remove(wktout)
-         file.remove(wktin)
+         if (lverbose)
+            message("'gdalsrsinfo' engine (read)")
+         if (FALSE) ## slow
+            grid$proj4 <- .gsub("\'","",system2("gdalsrsinfo"
+                                               ,c("-o proj4",wkt),stdout=TRUE,stderr=FALSE))
+         else {
+            tmp <- .maketmp()
+            wktin <- paste0(tmp,".prj~")
+            writeLines(wkt,wktin)
+            wktout <- paste0(tmp,".wkt~")
+            system2("gdalsrsinfo",c("-o proj4",wktin),stdout=wktout,stderr=FALSE)
+            grid$proj4 <- .gsub("\'","",readLines(wktout,warn=FALSE))
+            file.remove(wktout)
+            file.remove(wktin)
+            grid$proj4 <- grid$proj4[nchar(grid$proj4)>0]
+         }
       }
       else if (!("sf" %in% loadedNamespaces())) {
          if (lverbose)
